//Receive Unit
//Uses Bruce Land's Code to transmit to Matlab
//Uses Meghan Desai's Code to receive from RCR 433

//Yuchen Zhang yz238
//Richard Zhang rz44
//Dual-Channel Mobile EMG
//ECE 4760

//Includes
#include <avr/interrupt.h>
#include <avr/io.h>
#include "txrx.h"
#include <stdio.h>
#include "uart.h"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//definitions
#define begin {
#define end }
#define MAX_RX_LENGTH 128
#define threshold 2000

//declarations

// identify the assembler routine to C
extern int multfix(int a,int b);

char my_rx_data[MAX_RX_LENGTH];
volatile unsigned int time;
volatile char done, length, i, j, k;
volatile  char last_packet,new_packet;
volatile int lost_packets;
volatile char numChannels;

// TX ISR variables								
volatile char t_index; 		 //current string index
volatile char t_buffer[16];	//output string 
volatile char t_ready;  	//flag for transmit done
volatile char t_char; 		 //current character
volatile char newstr;

//**********************************************************
//timer 0 compare ISR
ISR (TIMER0_COMPA_vect)
begin      
	if(time<=threshold) time++; 
end  


//UART xmit-empty ISR
ISR (USART0_UDRE_vect)
{
	t_char = t_buffer[++t_index];
	if (t_index == 1) 			// } of string?	
	{
		UCSR0B ^= (1<<UDRIE0) ; 	// kill isr -- clear tx enable
		t_ready = 1; 			// transmit done 
	}
	else	UDR0 = t_char ;  	//s} the char 
}

//**********************************************************
//initialization
void init()
begin

	DDRB = 0xff;
	PORTB =0xff;
		
         //Used for LED array, output
	DDRC = 0xff;
	PORTC=0xff; //starting off

	//PORTD used for RX
	//d2 led, d0 RX, d1,d3..7 unused
	DDRD = 0xfe;
	PORTD&=(0<<2); //turn d2 led on as power-on led
                          
  TIMSK0 = 2;		//turn on timer 0 cmp match ISR 
  OCR0A =  250;  	//set the compare reg to 250 time ticks
  TCCR0A = 0b00000010; // turn on clear-on-match
  TCCR0B = 0b00000011;	// clock prescalar to 64


    //TXRX init
 	txrx_init(1,1,832,1);//TX/RX - 2400 baud - led on
    rx_reset(MAX_RX_LENGTH);


	stdout = stdin = stderr = &uart_str;
	fprintf(stdout,"Starting ISR UART demo...\n\r"); 
        
	//initialize variables
	time=0;
	done=0;

    sei();  
end                                                            

//**********************************************************
int main()
{

init();
last_packet = 255;
new_packet = 0;
lost_packets = 0;

while(1)
{ 
	if(rxdone()==1)
	{
        k=0;
        init_getrx();
        while(rx_empty()!=1)
        {       my_rx_data[k] = get_next_rx_data();
                k++;
        }
		rx_reset(MAX_RX_LENGTH); 

		new_packet = my_rx_data[4];

		if (last_packet > new_packet)//if we have run over
			lost_packets = new_packet - ((int)last_packet - 255);
		else
			lost_packets = new_packet - last_packet - 1;
		
        // 0 - 170
        // 1 - start_char
        // 2 - ID  
        // 3 - length  
        // 4 - number  
        // 5 to x - payload
		numChannels = decodeOne(my_rx_data[2]);//should be one or two

		if (numChannels == 1|| numChannels == 2 
							&& my_rx_data[3] == 31){//check for valid packet
        	
			
			fprintf(stdout,"%c%c%c",255,numChannels,lost_packets+1);//lost_packets+1 we subtract 1 in Matlab	
			//now transmit the current packet...
			PORTD ^=(1<<2);
		
			for (i = 5;i<35;i=i+5)
			{
				fprintf(stdout,"%c%c%c%c%c",my_rx_data[i],
					my_rx_data[i+1],my_rx_data[i+2],my_rx_data[i+3],my_rx_data[i+4]);
			}
			last_packet = my_rx_data[4];//the next last packet is this one...
		}                  
	}
    if(time==threshold)
	{
		time=0;
	}
} //while
return 0;
} //main


